//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2008 Sprite_tm
//fisier modificat de YO6PIR pentru Atmega8
//iesire PWM pe Timer2 

#include "io.h"
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//Remember: All signals are inverted due to the transistor-based
//5V->12V-conversion.
#define SHIFT_DATA 		PD4
#define SHIFT_STROBE	PD2
#define SHIFT_CLK		PD3
#define PWM_OUT			PB3		//pwm pe OC2
#define LED				PD5		//led control
#define Analog 			PD7

//Delay, in uS, to minimally make a pulse to the shiftregisters last
#define DELAY 5


void initio(void);
void setvfd(char digit, int data);
void handlepwm(void);

//Init all io-ports
void initio(void) {
    //Init I/O
    DDRD=(1<<SHIFT_DATA)|(1<<SHIFT_CLK)|(1<<SHIFT_STROBE)|(1<<LED);
    PORTD=0;
    DDRB=(1<<PWM_OUT);
    PORTB=0;
	//Seteaza Timer2 as FastPwm
    TCCR2=(1<<COM21)|(1<<WGM21)|(1<<WGM20)|(1<<CS20); //fast pwm on OCR2, CLK=T2S,NO prescaling
   // TCCR1B=(1<<CS10);
    OCR2=10;	//init OCR2=50% duty

    ACSR=(1<<ACBG); //enable AIN1<->vref-comparison
    //DIDR=(1<<AIN1D); //disable AIN1 digital pin
/*
    //seteaza USART pentru comunicatia seriala
    UCSRA=0;
    UCSRB=0x98; //enable tx, rx and rx int
    UCSRC=0x06; //8n1
    UBRRL=0x5; //115200 baud
*/
	//seteaza Timer1
    TCCR1A=0;
    TCCR1B=(1<<WGM12)|(1<<CS12)|(1<<CS10); //Prescaler=1024
    //Timer1 ticks at: 8000000/1024=7812 Hz.
  	OCR1A=2000; 					//Preset Timer1 la 0,3sec=> 7812 x 0.3 = 2343
    TIMSK=(1<<OCIE1A);	//Enable Timer1_isr si Timer1_ovf
    sei();							//Enable General Interrupts
}

//Boostconverter adjustments. Check analog comparator, adjust pwm-value on it.
#define STEPUP 1
#define STEPDOWN 1
void handlepwm(void) {
    int t=OCR2;
    if (ACSR&(1<<ACO)) {
	if (t<150) t+=STEPUP;
    } else {
	if (t>STEPDOWN) t-=STEPDOWN;
    }
    OCR2=t;
}

//Shift one bit into the shiftregister chain.
void shift(char bit) {
    if (bit) {
	PORTD&=~(1<<SHIFT_DATA);
    } else {
	PORTD|=(1<<SHIFT_DATA);
    }
    _delay_us(DELAY);
    PORTD&=~(1<<SHIFT_CLK);
    _delay_us(DELAY);
    PORTD|=(1<<SHIFT_CLK);
}

//Pulse strobe of the shiftregister-chain
void strobe(void) {
    PORTD&=~(1<<SHIFT_STROBE);
    _delay_us(DELAY);
    PORTD|=(1<<SHIFT_STROBE);
}

//Activeaza grilele pt.fiecare digit si seteaza segmentele 
//la valoarea inscrisa in data
void setvfd(char digit, int data) {
    char x;
    int b;

    //Prima data : trimite activarea <1> spre grile
    //O singura grila din 6 este selectata, restul in zero
    for (x=5; x>=0; x--) {
	if (x==digit) {
	    shift(1); 
	} else {
	    shift(0);
	}
    }
    
    //Second: Trimte datele catre segmente care formeaza litera
	//Fiecare litera are 14 biti
    b=1;
    for (x=0; x<13; x++) {
	if (data&b) {
	    shift(1);
	} else {
	    shift(0);
	}
	b<<=1;
    }
    
    //Finally: Trimite datele spre afisaj
    strobe();
}
